@@ -28,10 +28,27 @@ As an example, say you wanted to store shopping cart information, and you
  create a hash of it.  Then people can only use the urls you generated.
 
 Shopping cart data is a good example of something *NOT* to trust this with.
-"""
 
-import hashlib, hmac
-import random, time
+Attacks it protects against.
+    * basic tampering
+    * length extension attacks
+    * timing attacks (should be combined with rate limiting, and banning)
+    * magic missile
+
+
+TODO:
+* help make a 'truely random' long salty key.
+    The strength of a HMAC depends on the size, and randomness of the key.
+    If a key is for example '' or 'asdf' then brute forcing is much easier.
+    So we need to help people generate an appropriate key, 
+    and then store it for later safely (eg not in a world readable file).
+* address replayability. Replayability is required for some functions, but not others.
+* address timeouts.  Useful to make urls only valid for a short while.
+    * think of using UTC rather than local time, so different servers can be used.
+* fuzz tests 
+
+"""
+import hashlib, hmac, base64, random, time
 
 
 
@@ -64,18 +81,26 @@ def sign(values, secret_salt, keys = None, length_used = None):
 
     # NOTE: using hmac with sha1 http://en.wikipedia.org/wiki/HMAC
     try:
-        the_hash = hmac.new(secret_salt, msg, hashlib.sha1).hexdigest()
+        the_digest = hmac.new(secret_salt, msg, hashlib.sha1).digest()
     except TypeError:
+        # python3 compatible code.
         encoded_key = secret_salt.encode()
         encoded_msg = msg.encode()
-        the_hash = hmac.new(encoded_key, encoded_msg, hashlib.sha1).hexdigest()
+        the_digest = hmac.new(encoded_key, encoded_msg, hashlib.sha1).digest()
+
+    # remove the last character a '=' character.
+    the_hash = base64.urlsafe_b64encode(the_digest)[:-1].decode()
 
     if length_used is not None:
         the_hash = the_hash[:length_used]
 
     return the_hash
 
-def verify(values, secret_salt, hash, keys = None, length_used = None):
+def verify(values, secret_salt, hash, 
+           keys = None, 
+           length_used = None,
+           random_sleep = True,
+          ):
     """ verify(values, secret_salt, hash, 
                keys = None, length_used = None) -> Bool
         Returns True if the url keys/values match the given hash.
@@ -86,8 +111,12 @@ def verify(values, secret_salt, hash, keys = None, length_used = None):
         keys:        list of the key parts of '&key=value'
         length_used: if None use the full length of the hash.  If an 
                        integer, then use that length of the hash.
+        random_sleep:
     """
     the_hash = sign(values, secret_salt, keys, length_used)
+    #return hash == the_hash
+
+
 
     # Make sure that the comparison of the strings is a constant time
     #   algorithm to make signing a valid and invalid one similar.
@@ -95,37 +124,20 @@ def verify(values, secret_salt, hash, keys = None, length_used = None):
     # If it takes a different amount of time to compare part of a valid 
     #   string to part of an invalid string, then there may be issues.
 
-    if "add sleep to prevent timing attack":
-        time.sleep(random.randint(50,100) * 0.00001)
-        return hash == the_hash
-    else:
-
+    if random_sleep:
         # NOTE: doing a random sleep seems to make it pass the 'timing attack'
-        #  Whereas comparing character to character still seems to be similar.
+        #  Whereas comparing character to character still seems to have variance.
         #   I tested comparing character by character with the unit test.
 
-        #note, we must make sure they are both the same length.
-        if len(the_hash) is not len(hash):
-            return False
-        result = 0
-        for i in range(len(the_hash)):
-            result |= ord(the_hash[i]) ^ ord(hash[i])
-            # chew some random amount of time...
-            # should be more than it takes to compare one character.
-            #if 1:
-            #    time.sleep(random.random() * 0.00001)
-            #else:
-            #    # "cpu burn" is probably more appropriate where time.sleep can not be used.
-            #    for x in range(random.randint(0, 6)):
-            #        99 == 22
-
-        return result == 0
-
-
-
-
-
-
+        time.sleep(random.randint(50,100) * 0.00001)
 
+    #note, we must make sure they are both the same length.
+    if len(the_hash) is not len(hash):
+        return False
+    result = 0
+    for a,b in zip(the_hash, hash):
+        result |= ord(a) ^ ord(a)
+
+    return result == 0
 
 
