@@ -40,10 +40,10 @@ class TestHashUrl(unittest.TestCase):
         secret_salt = "thesepretz"
         keys = None
         hash = signed_url.sign(values, secret_salt, keys = keys)
-        self.assertEqual(hash, 'a50db7418ca441c14d71fc3e600bd242c53c2659')
+        self.assertEqual(hash, 'pQ23QYykQcFNcfw-YAvSQsU8Jlk')
         # different secret_salt, different hash.
         hash2 = signed_url.sign(values, 'differentsecret_salt', keys = None)
-        self.assertEqual(hash2, 'fe64dfc8c77986937dc52fbe3de0e22258660b14')
+        self.assertEqual(hash2, '_mTfyMd5hpN9xS--PeDiIlhmCxQ')
 
         # see that the hash is ok.
         self.assertTrue( signed_url.verify(values, secret_salt, hash, keys=keys) )
@@ -67,11 +67,6 @@ class TestHashUrl(unittest.TestCase):
         secret_salt = "thesepretz"
         keys = None
         hash = signed_url.sign(values, secret_salt, keys = keys)
-        self.assertEqual(hash, 'a50db7418ca441c14d71fc3e600bd242c53c2659')
-        # different secret_salt, different hash.
-        hash2 = signed_url.sign(values, 'differentsecret_salt', keys = None)
-        self.assertEqual(hash2, 'fe64dfc8c77986937dc52fbe3de0e22258660b14')
-
         # see that the hash is ok.
         self.assertTrue( signed_url.verify(values, secret_salt, hash, keys=keys) )
         
